/*
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 *
 * This file is generated automatically from 'src/pmc/complex.pmc'
 * by tools/build/pmc2c.pl.
 *
 * Any changes made here will be lost!
 *
 */

#line 1 "src/pmc/complex.pmc"
/*
Copyright: 2004-2006 The Perl Foundation.  All Rights Reserved.
$Id: complex.pmc 12402 2006-04-23 20:44:17Z jonathan $

=head1 NAME

src/pmc/complex.pmc - Complex Numbers PMC Class

=head1 DESCRIPTION

C<Complex> provides a representation of complex numbers. It handles
string parsing/generating and basic mathematical operations.

=head2 Functions

=over 4

=cut

Equations used are sometimes listed.  At times, multiple equations are given,
but those starting with => are the onces used

*/

#include "parrot/parrot.h"


#define RE(obj) (((FLOATVAL*)PMC_struct_val(obj))[0])
#define IM(obj) (((FLOATVAL*)PMC_struct_val(obj))[1])


/*

=item C<static void
complex_parse_string(Interp* interpreter,
        FLOATVAL *re, FLOATVAL *im, STRING *value)>

Parses the string in C<value> to produce a complex number, represented
by the real (C<*re>) and imaginary (C<*im>) parts. Raises an exception
if it cannot understand the string.  The string should be of the form
C<a+bi> with optional spaces around C<+> and before C<i>. You can also
use C<j> instead of C<i>.

=cut

*/

static void
complex_parse_string(Interp* interpreter,
        FLOATVAL *re, FLOATVAL *im, STRING *value) {
    char *str;
    INTVAL first_num_length, second_num_length;
    INTVAL first_num_minus, second_num_minus, i;
    char *first_num_offset, *second_num_offset = NULL, *t;
    STRING *S;

    t = str = string_to_cstring(interpreter, value);
    i = 0;
    first_num_offset = str;
    first_num_minus = second_num_minus = 0;

    /* walk the string and identify the real and imaginary parts */

    if(*t == '-') {
        /* first number is negative */
        t++;
        first_num_minus = 1;
        if(*t == ' ') t++; /* allow for an optional space */
        first_num_offset = t;
    }
    while(*t >= '0' && *t <= '9') t++; /* skip digits */
    if(*t == '.') {
        /* this number has a decimal point */
        t++;
        while(*t >= '0' && *t <= '9') t++; /* skip digits */
    }
    /* save the length of the real part */
    first_num_length = t - first_num_offset;

    if(*t == 0) {
        /* end of string; we only have a real part */
        second_num_length = 0;
    }
    else if( (*t == 'i' || *t == 'j') && *(t+1) == 0 ) {
        /* there is only an imaginary part, so the first number was
            actually the imaginary part */
        second_num_length = first_num_length;
        first_num_length = 0;
        second_num_offset = first_num_offset;
        second_num_minus = first_num_minus;
        first_num_minus = 0;
        /* this is useful if there is no number for
            the imaginary part, like in "-i" */
        i = 1;
    }
    else {
        if(*t == ' ') t++; /* skip an optional space */
        /* expect "+" or "-" and the imaginary part */
        if(*t == '+' || *t == '-') {
            second_num_minus = (*t == '-'); /* save the sign */
            t++;
            if(*t == ' ') t++; /* skip another optional space */

            /* save the beginning of the imaginary part */
            second_num_offset = t;
            while(*t >= '0' && *t <= '9') t++; /* skip digits */
            if(*t == '.') {
                /* this number has a decimal point */
                t++;
                while(*t >= '0' && *t <= '9') t++; /* skip digits */
            }
            /* save the length of the imaginary part */
            second_num_length = t - second_num_offset;

            if(*t == ' ') t++; /* allow for one more optional space */

            /* verify that the string ends properly */
            if( (*t != 'i' && *t != 'j') || (*(t+1) != 0) ) {
                /* imaginary part does not end in 'i' or 'j' */
                real_exception(interpreter, NULL, E_ValueError,
                    "Complex: malformed string");
                first_num_length = second_num_length = 0;
            }
            /* this is useful if there is no number for the
                imaginary part, like in "2+i" */
            i = 1;

            /* all is OK, save the number */
        }
        else {
            /* "+" or "-" not found: error */
            real_exception(interpreter, NULL, E_ValueError,
                "Complex: malformed string");
            first_num_length = second_num_length = 0;
        }
    }

    /* now we have the offsets and the lengths;
        we turn them into float values */

    if(first_num_length) {
        /* there is a real part, interpret it */
        S = string_from_cstring(interpreter,
                first_num_offset, first_num_length);
        *re = string_to_num(interpreter, S);
    }
    else {
        /* consider the real part 0.0 */
        *re = 0.0;
    }

    if(second_num_length) {
        /* there is an imaginary part, interpret it */
        S = string_from_cstring(interpreter,
                second_num_offset, second_num_length);
        *im = string_to_num(interpreter, S);
    }
    else {
        /* consider the imaginary part 0.0 */
        if(i) /* the string was something like "1+i" */
            *im = 1.0;
        else
            *im = 0.0;
    }
    if(first_num_minus) *re = - *re;
    if(second_num_minus) *im = - *im;

    string_cstring_free(str);
}

/*

=item C<static FLOATVAL*
complex_locate_keyed_num(Interp* interpreter, PMC* self, STRING *key)>

Interpret the string C<key>; valid keys are C<real> and C<imag>,
representing the real and imaginary parts of the complex number.

=cut

*/

static FLOATVAL*
complex_locate_keyed_num(Interp* interpreter, PMC* self, STRING *key) {
    /* do imag first since real can be read much faster anyway */
    STRING *imag = string_from_cstring(interpreter, "imag", 4);
    STRING *real;

    if(0 == string_equal(interpreter, key, imag))
        return &IM(self);
    real = string_from_cstring(interpreter, "real", 4);
    if(0 == string_equal(interpreter, key, real))
        return &RE(self);
    real_exception(interpreter, NULL, E_KeyError,
        "Complex: key is neither 'real' or 'imag'");
    return NULL;
}

static void
int_check_divide_zero(Interp *interpreter, INTVAL value)
{
    if (value == 0) {
        real_exception(interpreter, NULL, E_ZeroDivisionError,
                       "Divide by zero");
    }
}

static void
float_check_divide_zero(Interp *interpreter, FLOATVAL value)
{
    if (value == 0.0) {
        real_exception(interpreter, NULL, E_ZeroDivisionError,
                       "Divide by zero");
    }
}

static void
complex_check_divide_zero(Interp *interpreter, PMC *value)
{
    /* Throw an exception if we are dividing by zero. */
    if (RE(value) == 0.0 && IM(value) == 0.0) {
        real_exception(interpreter, NULL, E_ZeroDivisionError,
                       "Divide by zero");
    }
}


#line 240 "src/pmc/complex.c"
#include "parrot/parrot.h"
#include "parrot/extend.h"
#include "parrot/dynext.h"
#include "pmc_complex.h"
#include "pmc_default.h"
#include "complex.str"

void
Parrot_Complex_init(Interp* interpreter, PMC* pmc)
#line 338 "src/pmc/complex.pmc"
{
    /* XXX should check if mem_sys_allocate failed */
    FLOATVAL* data = (FLOATVAL*)mem_sys_allocate(2 * sizeof(FLOATVAL));
    PMC_struct_val(pmc) = data;
    PObj_active_destroy_SET(pmc);
    RE(pmc) = IM(pmc) = 0.0;

}
#line 259 "src/pmc/complex.c"


void
Parrot_Complex_init_pmc(Interp* interpreter, PMC* pmc, PMC* initializer)
#line 346 "src/pmc/complex.pmc"
{
    /* XXX not implemented */
     VTABLE_init(interpreter, pmc);

}
#line 270 "src/pmc/complex.c"


PMC*
Parrot_Complex_instantiate(Interp* interpreter, PMC* pmc, PMC* sig)
#line 240 "src/pmc/complex.pmc"
{
    PMC *res = pmc_new(interpreter, enum_class_Complex);
    FLOATVAL re = 0.0, im = 0.0;
    int argcI = REG_INT(1);
    int argcS = REG_INT(2);
    int argcP = REG_INT(3);
    int argcN = REG_INT(4);

    return PMCNULL;  /* TODO */
    /*
     * we can only allow 0..2 arguments of one kind. For
     * mixed (e.g. N,P) args the order of arguments isn't fixed
     */
    if (argcP) {
        re = VTABLE_get_number(interpreter, REG_PMC(5));
        if (argcP == 2)
            im = VTABLE_get_number(interpreter, REG_PMC(6));
        /*
         * TODO throw exception if argument mismatch
         */
    }
    else if (argcN) {
        re = REG_NUM(5);
        if (argcN == 2)
            im = REG_NUM(6);
    }
    else if (argcI) {
        re = REG_INT(5);
        if (argcI == 2)
            im = REG_INT(6);
    }
    else if (argcS == 1) {
        complex_parse_string(interpreter, &RE(res), &IM(res), REG_STR(5));
        return res;
    }
    RE(res) = re;
    IM(res) = im;
    return res;

}
#line 316 "src/pmc/complex.c"


void
Parrot_Complex_morph(Interp* interpreter, PMC* pmc, INTVAL type)
#line 356 "src/pmc/complex.pmc"
{
    if (pmc->vtable->base_type == type)
        return;
    Parrot_default_morph(interpreter, pmc, type);

}
#line 328 "src/pmc/complex.c"


void
Parrot_Complex_destroy(Interp* interpreter, PMC* pmc)
#line 351 "src/pmc/complex.pmc"
{
    mem_sys_free(PMC_struct_val(pmc));
    PMC_struct_val(pmc) = NULL;

}
#line 339 "src/pmc/complex.c"


PMC*
Parrot_Complex_clone(Interp* interpreter, PMC* pmc)
#line 362 "src/pmc/complex.pmc"
{
    PMC* dest = pmc_new_noinit(interpreter, pmc->vtable->base_type);
    FLOATVAL* data = (FLOATVAL*)mem_sys_allocate(2 * sizeof(FLOATVAL));
    PMC_struct_val(dest) = data;
    PObj_active_destroy_SET(dest);
    RE(dest) = RE(pmc);
    IM(dest) = IM(pmc);
    return dest;

}
#line 355 "src/pmc/complex.c"


INTVAL
Parrot_Complex_get_integer(Interp* interpreter, PMC* pmc)
#line 394 "src/pmc/complex.pmc"
{
    return (INTVAL)( VTABLE_get_number(interpreter, pmc));

}
#line 365 "src/pmc/complex.c"


INTVAL
Parrot_Complex_get_integer_keyed(Interp* interpreter, PMC* pmc, PMC* key)
#line 433 "src/pmc/complex.pmc"
{
    STRING* s = VTABLE_get_string(interpreter, key);
    return  VTABLE_get_integer_keyed_str(interpreter, pmc, s);

}
#line 376 "src/pmc/complex.c"


INTVAL
Parrot_Complex_get_integer_keyed_str(Interp* interpreter, PMC* pmc, STRING* key)
#line 438 "src/pmc/complex.pmc"
{
    return (INTVAL)( VTABLE_get_number_keyed_str(interpreter, pmc, key));

}
#line 386 "src/pmc/complex.c"


FLOATVAL
Parrot_Complex_get_number(Interp* interpreter, PMC* pmc)
#line 398 "src/pmc/complex.pmc"
{
    return sqrt(RE(pmc)*RE(pmc) + IM(pmc)*IM(pmc));

}
#line 396 "src/pmc/complex.c"


FLOATVAL
Parrot_Complex_get_number_keyed(Interp* interpreter, PMC* pmc, PMC* key)
#line 442 "src/pmc/complex.pmc"
{
    STRING* s = VTABLE_get_string(interpreter, key);
    return  VTABLE_get_number_keyed_str(interpreter, pmc, s);

}
#line 407 "src/pmc/complex.c"


FLOATVAL
Parrot_Complex_get_number_keyed_int(Interp* interpreter, PMC* pmc, INTVAL key)
#line 506 "src/pmc/complex.pmc"
{
    switch (key) {
        case 0:
            return RE(pmc);
        case 1:
            return IM(pmc);
        default:
            real_exception(interpreter, NULL, E_KeyError,
                    "Complex: key must be 0 or 1");
    }
    return 0.0;

}
#line 426 "src/pmc/complex.c"


FLOATVAL
Parrot_Complex_get_number_keyed_str(Interp* interpreter, PMC* pmc, STRING* key)
#line 447 "src/pmc/complex.pmc"
{
    FLOATVAL *num = complex_locate_keyed_num(interpreter, pmc, key);
    if(num)
        return *num;
    return 0.0;

}
#line 439 "src/pmc/complex.c"


STRING*
Parrot_Complex_get_string(Interp* interpreter, PMC* pmc)
#line 402 "src/pmc/complex.pmc"
{
    STRING *s;
        s = Parrot_sprintf_c(interpreter,
                "%vg%+vgi", RE(pmc), IM(pmc));
    return s;

}
#line 452 "src/pmc/complex.c"


INTVAL
Parrot_Complex_get_bool(Interp* interpreter, PMC* pmc)
#line 409 "src/pmc/complex.pmc"
{
    return (INTVAL)(RE(pmc) != 0.0 || IM(pmc) != 0.0);

}
#line 462 "src/pmc/complex.c"


PMC*
Parrot_Complex_get_pmc_keyed(Interp* interpreter, PMC* pmc, PMC* key)
#line 454 "src/pmc/complex.pmc"
{
    STRING* s = VTABLE_get_string(interpreter, key);
    return  VTABLE_get_pmc_keyed_str(interpreter, pmc, s);

}
#line 473 "src/pmc/complex.c"


PMC*
Parrot_Complex_get_pmc_keyed_int(Interp* interpreter, PMC* pmc, INTVAL key)
#line 479 "src/pmc/complex.pmc"
{
    PMC *ret;
    FLOATVAL val;

    ret = pmc_new(interpreter, enum_class_Float);
    val =  VTABLE_get_number_keyed_int(interpreter, pmc, key);
    VTABLE_set_number_native(interpreter, ret, val);
    return ret;

}
#line 489 "src/pmc/complex.c"


PMC*
Parrot_Complex_get_pmc_keyed_str(Interp* interpreter, PMC* pmc, STRING* key)
#line 459 "src/pmc/complex.pmc"
{
    PMC *ret;
    FLOATVAL val;

    ret = pmc_new(interpreter, enum_class_Float);
    val =  VTABLE_get_number_keyed_str(interpreter, pmc, key);
    VTABLE_set_number_native(interpreter, ret, val);
    return ret;

}
#line 505 "src/pmc/complex.c"


void
Parrot_Complex_set_integer_native(Interp* interpreter, PMC* pmc, INTVAL value)
#line 568 "src/pmc/complex.pmc"
{
     VTABLE_set_number_native(interpreter, pmc, value);

}
#line 515 "src/pmc/complex.c"


void
Parrot_Complex_set_integer_keyed(Interp* interpreter, PMC* pmc, PMC* key, INTVAL value)
#line 598 "src/pmc/complex.pmc"
{
     VTABLE_set_number_keyed(interpreter, pmc, key, value);

}
#line 525 "src/pmc/complex.c"


void
Parrot_Complex_set_integer_keyed_str(Interp* interpreter, PMC* pmc, STRING* key, INTVAL value)
#line 602 "src/pmc/complex.pmc"
{
     VTABLE_set_number_keyed_str(interpreter, pmc, key, value);

}
#line 535 "src/pmc/complex.c"


void
Parrot_Complex_set_number_native(Interp* interpreter, PMC* pmc, FLOATVAL value)
#line 572 "src/pmc/complex.pmc"
{
    RE(pmc) = value;
    IM(pmc) = 0.0;

}
#line 546 "src/pmc/complex.c"


void
Parrot_Complex_set_number_keyed(Interp* interpreter, PMC* pmc, PMC* key, FLOATVAL value)
#line 606 "src/pmc/complex.pmc"
{
    STRING* s = VTABLE_get_string(interpreter, key);
     VTABLE_set_number_keyed_str(interpreter, pmc, s, value);

}
#line 557 "src/pmc/complex.c"


void
Parrot_Complex_set_number_keyed_int(Interp* interpreter, PMC* pmc, INTVAL key, FLOATVAL v)
#line 519 "src/pmc/complex.pmc"
{
    switch (key) {
        case 0:
            RE(pmc) = v;
            break;
        case 1:
            IM(pmc) = v;
            break;
        default:
            real_exception(interpreter, NULL, E_KeyError,
                    "Complex: key must be 0 or 1");
    }

}
#line 577 "src/pmc/complex.c"


void
Parrot_Complex_set_number_keyed_str(Interp* interpreter, PMC* pmc, STRING* key, FLOATVAL value)
#line 611 "src/pmc/complex.pmc"
{
    FLOATVAL *num = complex_locate_keyed_num(interpreter, pmc, key);
    if(num)
        *num = value;

}
#line 589 "src/pmc/complex.c"


void
Parrot_Complex_set_string_native(Interp* interpreter, PMC* pmc, STRING* value)
#line 554 "src/pmc/complex.pmc"
{
    complex_parse_string(interpreter, &RE(pmc), &IM(pmc), value);

}
#line 599 "src/pmc/complex.c"


void
Parrot_Complex_set_pmc(Interp* interpreter, PMC* pmc, PMC* value)
#line 558 "src/pmc/complex.pmc"
{
    if(value->vtable->base_type == enum_class_Complex) {
        RE(pmc) = RE(value);
        IM(pmc) = IM(value);
    }
    else {
         VTABLE_set_string_native(interpreter, pmc, VTABLE_get_string(interpreter, value));
    }

}
#line 615 "src/pmc/complex.c"


void
Parrot_Complex_set_pmc_keyed(Interp* interpreter, PMC* pmc, PMC* key, PMC* value)
#line 617 "src/pmc/complex.pmc"
{
    FLOATVAL f = VTABLE_get_number(interpreter, value);
     VTABLE_set_number_keyed(interpreter, pmc, key, f);

}
#line 626 "src/pmc/complex.c"


void
Parrot_Complex_set_pmc_keyed_str(Interp* interpreter, PMC* pmc, STRING* key, PMC* value)
#line 622 "src/pmc/complex.pmc"
{
    FLOATVAL f = VTABLE_get_number(interpreter, value);
     VTABLE_set_number_keyed_str(interpreter, pmc, key, f);

}
#line 637 "src/pmc/complex.c"


PMC*
Parrot_Complex_add(Interp* interpreter, PMC* pmc, PMC* value, PMC* dest)
#line 641 "src/pmc/complex.pmc"
{
        if (dest)
            VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interpreter, pmc->vtable->base_type);
        }
        RE(dest) = RE(pmc) + VTABLE_get_number(interpreter, value);
        IM(dest) = IM(pmc);
        return dest;
    
}
#line 654 "src/pmc/complex.c"
PMC*
Parrot_Complex_add_Complex(Interp* interpreter, PMC* pmc, PMC* value, PMC* dest)
{
        if (dest)
            VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interpreter, pmc->vtable->base_type);
        }
        RE(dest) = RE(pmc) + RE(value);
        IM(dest) = IM(pmc) + IM(value);
        return dest;
    
}

PMC*
Parrot_Complex_add_int(Interp* interpreter, PMC* pmc, INTVAL value, PMC* dest)
#line 664 "src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interpreter, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) + value;
    IM(dest) = IM(pmc);
    return dest;

}
#line 683 "src/pmc/complex.c"


PMC*
Parrot_Complex_add_float(Interp* interpreter, PMC* pmc, FLOATVAL value, PMC* dest)
#line 675 "src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interpreter, pmc->vtable->base_type);
    }
    VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
    RE(dest) = RE(pmc) + value;
    IM(dest) = IM(pmc);
    return dest;

}
#line 701 "src/pmc/complex.c"


void
Parrot_Complex_i_add(Interp* interpreter, PMC* pmc, PMC* value)
#line 687 "src/pmc/complex.pmc"
{
        RE(pmc) += VTABLE_get_number(interpreter, value);
    
}
#line 711 "src/pmc/complex.c"
void
Parrot_Complex_i_add_Complex(Interp* interpreter, PMC* pmc, PMC* value)
{
        RE(pmc) += RE(value);
        IM(pmc) += IM(value);
    
}

void
Parrot_Complex_i_add_int(Interp* interpreter, PMC* pmc, INTVAL value)
#line 697 "src/pmc/complex.pmc"
{
    RE(pmc) += (FLOATVAL)value;

}
#line 727 "src/pmc/complex.c"


void
Parrot_Complex_i_add_float(Interp* interpreter, PMC* pmc, FLOATVAL value)
#line 700 "src/pmc/complex.pmc"
{
    RE(pmc) += value;

}
#line 737 "src/pmc/complex.c"


PMC*
Parrot_Complex_subtract(Interp* interpreter, PMC* pmc, PMC* value, PMC* dest)
#line 718 "src/pmc/complex.pmc"
{
        if (dest)
            VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interpreter, pmc->vtable->base_type);
        }
        RE(dest) = RE(pmc) - VTABLE_get_number(interpreter, value);
        IM(dest) = IM(pmc);
        return dest;
    
}
#line 754 "src/pmc/complex.c"
PMC*
Parrot_Complex_subtract_Complex(Interp* interpreter, PMC* pmc, PMC* value, PMC* dest)
{
        if (dest)
            VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interpreter, pmc->vtable->base_type);
        }
        RE(dest) = RE(pmc) - RE(value);
        IM(dest) = IM(pmc) - IM(value);
        return dest;
    
}

PMC*
Parrot_Complex_subtract_int(Interp* interpreter, PMC* pmc, INTVAL value, PMC* dest)
#line 741 "src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interpreter, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) - value;
    IM(dest) = IM(pmc);
    return dest;

}
#line 783 "src/pmc/complex.c"


PMC*
Parrot_Complex_subtract_float(Interp* interpreter, PMC* pmc, FLOATVAL value, PMC* dest)
#line 752 "src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interpreter, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) - value;
    IM(dest) = IM(pmc);
    return dest;

}
#line 800 "src/pmc/complex.c"


void
Parrot_Complex_i_subtract(Interp* interpreter, PMC* pmc, PMC* value)
#line 763 "src/pmc/complex.pmc"
{
        RE(pmc) -= VTABLE_get_number(interpreter, value);
    
}
#line 810 "src/pmc/complex.c"
void
Parrot_Complex_i_subtract_Complex(Interp* interpreter, PMC* pmc, PMC* value)
{
        RE(pmc) -= RE(value);
        IM(pmc) -= IM(value);
    
}

void
Parrot_Complex_i_subtract_int(Interp* interpreter, PMC* pmc, INTVAL value)
#line 773 "src/pmc/complex.pmc"
{
    RE(pmc) -= (FLOATVAL)value;

}
#line 826 "src/pmc/complex.c"


void
Parrot_Complex_i_subtract_float(Interp* interpreter, PMC* pmc, FLOATVAL value)
#line 776 "src/pmc/complex.pmc"
{
    RE(pmc) -= value;

}
#line 836 "src/pmc/complex.c"


PMC*
Parrot_Complex_multiply(Interp* interpreter, PMC* pmc, PMC* value, PMC* dest)
#line 809 "src/pmc/complex.pmc"
{
        FLOATVAL re = RE(pmc) * VTABLE_get_number(interpreter, value);
        FLOATVAL im = IM(pmc) * VTABLE_get_number(interpreter, value);
        if (dest)
            VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interpreter, pmc->vtable->base_type);
        }
        RE(dest) = re;
        IM(dest) = im;
        return dest;
    
}
#line 855 "src/pmc/complex.c"
PMC*
Parrot_Complex_multiply_Complex(Interp* interpreter, PMC* pmc, PMC* value, PMC* dest)
{
        FLOATVAL re = RE(pmc) * RE(value) - IM(pmc) * IM(value);
        FLOATVAL im = IM(pmc) * RE(value) + RE(pmc) * IM(value);
        if (dest)
            VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interpreter, pmc->vtable->base_type);
        }
        RE(dest) = re;
        IM(dest) = im;
        return dest;
    
}

PMC*
Parrot_Complex_multiply_int(Interp* interpreter, PMC* pmc, INTVAL value, PMC* dest)
#line 836 "src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interpreter, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) * value;
    IM(dest) = IM(pmc) * value;
    return dest;

}
#line 886 "src/pmc/complex.c"


PMC*
Parrot_Complex_multiply_float(Interp* interpreter, PMC* pmc, FLOATVAL value, PMC* dest)
#line 847 "src/pmc/complex.pmc"
{
    if (dest)
        VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interpreter, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) * value;
    IM(dest) = IM(pmc) * value;
    return dest;

}
#line 903 "src/pmc/complex.c"


void
Parrot_Complex_i_multiply(Interp* interpreter, PMC* pmc, PMC* value)
#line 858 "src/pmc/complex.pmc"
{
        RE(pmc) *= VTABLE_get_number(interpreter, value);
        IM(pmc) *= VTABLE_get_number(interpreter, value);
    
}
#line 914 "src/pmc/complex.c"
void
Parrot_Complex_i_multiply_Complex(Interp* interpreter, PMC* pmc, PMC* value)
{
        FLOATVAL re = RE(pmc) * RE(value) - IM(pmc) * IM(value);
        FLOATVAL im = IM(pmc) * RE(value) + RE(pmc) * IM(value);
        RE(pmc) = re;
        IM(pmc) = im;
    
}

void
Parrot_Complex_i_multiply_int(Interp* interpreter, PMC* pmc, INTVAL value)
#line 871 "src/pmc/complex.pmc"
{
    RE(pmc) *= value;
    IM(pmc) *= value;

}
#line 933 "src/pmc/complex.c"


void
Parrot_Complex_i_multiply_float(Interp* interpreter, PMC* pmc, FLOATVAL value)
#line 876 "src/pmc/complex.pmc"
{
    RE(pmc) *= value;
    IM(pmc) *= value;

}
#line 944 "src/pmc/complex.c"


PMC*
Parrot_Complex_divide(Interp* interpreter, PMC* pmc, PMC* value, PMC* dest)
#line 911 "src/pmc/complex.pmc"
{
        FLOATVAL d = VTABLE_get_number(interpreter, value);
        float_check_divide_zero(interpreter, d);
        if (dest)
            VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interpreter, pmc->vtable->base_type);
        }
        RE(dest) = RE(pmc) / d;
        IM(dest) = IM(pmc) / d;
        return dest;
    
}
#line 963 "src/pmc/complex.c"
PMC*
Parrot_Complex_divide_Complex(Interp* interpreter, PMC* pmc, PMC* value, PMC* dest)
{
        FLOATVAL mod, re, im;

        complex_check_divide_zero(interpreter, value);
        if (dest)
            VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
        else {
            dest = pmc_new(interpreter, pmc->vtable->base_type);
        }
        /* a little speed optimisation: cache an intermediate number;
            I'm not sure the compiler does this */
        mod = (RE(value) * RE(value) + IM(value) * IM(value));
        re = (RE(pmc) * RE(value) + IM(pmc) * IM(value)) / mod;
        im = (IM(pmc) * RE(value) - RE(pmc) * IM(value)) / mod;
        RE(dest) = re;
        IM(dest) = im;
        return dest;
    
}

PMC*
Parrot_Complex_divide_int(Interp* interpreter, PMC* pmc, INTVAL value, PMC* dest)
#line 944 "src/pmc/complex.pmc"
{
    int_check_divide_zero(interpreter, value);
    if (dest)
        VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interpreter, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) / value;
    IM(dest) = IM(pmc) / value;
    return dest;

}
#line 1001 "src/pmc/complex.c"


PMC*
Parrot_Complex_divide_float(Interp* interpreter, PMC* pmc, FLOATVAL value, PMC* dest)
#line 956 "src/pmc/complex.pmc"
{
    float_check_divide_zero(interpreter, value);
    if (dest)
        VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
    else {
        dest = pmc_new(interpreter, pmc->vtable->base_type);
    }
    RE(dest) = RE(pmc) / value;
    IM(dest) = IM(pmc) / value;
    return dest;

}
#line 1019 "src/pmc/complex.c"


void
Parrot_Complex_i_divide(Interp* interpreter, PMC* pmc, PMC* value)
#line 968 "src/pmc/complex.pmc"
{
        FLOATVAL d = VTABLE_get_number(interpreter, value);
        float_check_divide_zero(interpreter, d);
        RE(pmc) /= d;
        IM(pmc) /= d;
    
}
#line 1032 "src/pmc/complex.c"
void
Parrot_Complex_i_divide_Complex(Interp* interpreter, PMC* pmc, PMC* value)
{
        FLOATVAL mod, re, im;

        complex_check_divide_zero(interpreter, value);
        /* a little speed optimisation: cache an intermediate number;
            I'm not sure the compiler does this */
        mod = (RE(value) * RE(value) + IM(value) * IM(value));
        re = (RE(pmc) * RE(value) + IM(pmc) * IM(value)) / mod;
        im = (IM(pmc) * RE(value) - RE(pmc) * IM(value)) / mod;
        RE(pmc) = re;
        IM(pmc) = im;
    
}

void
Parrot_Complex_i_divide_int(Interp* interpreter, PMC* pmc, INTVAL value)
#line 989 "src/pmc/complex.pmc"
{
    int_check_divide_zero(interpreter, value);
    RE(pmc) /= value;
    IM(pmc) /= value;

}
#line 1058 "src/pmc/complex.c"


void
Parrot_Complex_i_divide_float(Interp* interpreter, PMC* pmc, FLOATVAL value)
#line 995 "src/pmc/complex.pmc"
{
    float_check_divide_zero(interpreter, value);
    RE(pmc) /= value;
    IM(pmc) /= value;

}
#line 1070 "src/pmc/complex.c"


PMC*
Parrot_Complex_pow(Interp* interpreter, PMC* pmc, PMC* value, PMC* dest)
#line 1575 "src/pmc/complex.pmc"
{
             PMC *l = pmc_new(interpreter, pmc->vtable->base_type);
             if (dest)
                 VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
             else
                 dest = pmc_new(interpreter, pmc->vtable->base_type);
             l = Parrot_Complex_multiply(interpreter,
                     Parrot_Complex_ln(interpreter, pmc), value, l);
             dest = Parrot_Complex_exp(interpreter, l);
             return dest;
         
}
#line 1088 "src/pmc/complex.c"
PMC*
Parrot_Complex_pow_Complex(Interp* interpreter, PMC* pmc, PMC* value, PMC* dest)
{
             PMC *l = pmc_new(interpreter, pmc->vtable->base_type);
             if (dest)
                 VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
             else
                 dest = pmc_new(interpreter, pmc->vtable->base_type);
             l = Parrot_Complex_multiply_Complex(interpreter,
                     Parrot_Complex_ln(interpreter, pmc), value, l);
             dest = Parrot_Complex_exp(interpreter, l);
             return dest;
         
}

PMC*
Parrot_Complex_absolute(Interp* interpreter, PMC* pmc, PMC *dest)
#line 1077 "src/pmc/complex.pmc"
{
    FLOATVAL d = sqrt(RE(pmc)*RE(pmc) + IM(pmc)*IM(pmc));
    if (!dest)
        dest = pmc_new(interpreter,
            Parrot_get_ctx_HLL_type(interpreter, enum_class_Float));
    VTABLE_set_number_native(interpreter, dest, d);
    return dest;

}
#line 1116 "src/pmc/complex.c"


void
Parrot_Complex_i_absolute(Interp* interpreter, PMC* pmc)
#line 1086 "src/pmc/complex.pmc"
{
    FLOATVAL d = sqrt(RE(pmc)*RE(pmc) + IM(pmc)*IM(pmc));
    VTABLE_morph(interpreter, pmc, enum_class_Float);   /* XXX */
    VTABLE_set_number_native(interpreter, pmc, d);

}
#line 1128 "src/pmc/complex.c"


PMC*
Parrot_Complex_neg(Interp* interpreter, PMC* pmc, PMC* dest)
#line 1014 "src/pmc/complex.pmc"
{
    if (!dest)
        dest = pmc_new(interpreter, pmc->vtable->base_type);
    else
        VTABLE_morph(interpreter, dest, pmc->vtable->base_type);
    RE(dest) = - RE(pmc);
    IM(dest) = - IM(pmc);
    return dest;

}
#line 1144 "src/pmc/complex.c"


void
Parrot_Complex_i_neg(Interp* interpreter, PMC* pmc)
#line 1024 "src/pmc/complex.pmc"
{
    RE(pmc) = - RE(pmc);
    IM(pmc) = - IM(pmc);

}
#line 1155 "src/pmc/complex.c"


INTVAL
Parrot_Complex_is_equal(Interp* interpreter, PMC* pmc, PMC* value)
#line 1039 "src/pmc/complex.pmc"
{
        if(IM(pmc) != 0.0)
            return (INTVAL)0;
        return (RE(pmc) == VTABLE_get_number(interpreter, value));
    
}
#line 1167 "src/pmc/complex.c"
INTVAL
Parrot_Complex_is_equal_Complex(Interp* interpreter, PMC* pmc, PMC* value)
{
        return (INTVAL)(
                RE(pmc) == RE(value) &&
                IM(pmc) == IM(value)
            );
    
}

void*
Parrot_Complex_invoke(Interp* interpreter, PMC* pmc, void* next)
#line 290 "src/pmc/complex.pmc"
{
    int argcP = REG_INT(3);
    PMC *res = pmc_new(interpreter, enum_class_Complex);
    if (argcP == 1) {
        PMC *arg = REG_PMC(5);
        if (arg->vtable->base_type == enum_class_String) {
            VTABLE_set_string_native(interpreter, res, PMC_str_val(arg));
        }
        else
            RE(res) = VTABLE_get_number(interpreter, arg);
    }
    else if (argcP == 2) {
        RE(res) = VTABLE_get_number(interpreter, REG_PMC(5));
        IM(res) = VTABLE_get_number(interpreter, REG_PMC(6));
    }
    REG_PMC(5) = res;
    return next;

}
#line 1200 "src/pmc/complex.c"


PMC*
Parrot_Complex_ln(Interp* interpreter, PMC* pmc)
#line 1113 "src/pmc/complex.pmc"
{
    FLOATVAL re, im;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    re = RE(pmc);
    im = IM(pmc);
    /* This is necessary for atan2 to behave */
    if (im == -0.0) im = 0.0;
    RE(d) = log(sqrt(re*re + im*im));
    IM(d) = atan2(im, re);
    return d;

}
#line 1218 "src/pmc/complex.c"


PMC*
Parrot_Complex_exp(Interp* interpreter, PMC* pmc)
#line 1137 "src/pmc/complex.pmc"
{
    FLOATVAL re, im, f;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    re = RE(pmc);
    im = IM(pmc);
    f = exp(re);
    RE(d) = f * cos(im);
    /* If only sin(pi) worked. */
    if (im == 4.0 * atan(1.0))
        IM(d) = 0.0;
    else
        IM(d) = f * sin(im);
    return d;

}
#line 1239 "src/pmc/complex.c"


PMC*
Parrot_Complex_sin(Interp* interpreter, PMC* pmc)
#line 1185 "src/pmc/complex.pmc"
{
    FLOATVAL re, im;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    re = RE(pmc);
    im = IM(pmc);
    if (im == 0.0) {
        RE(d) = sin(re);
        IM(d) = 0.0;
        return d;
    }
    if (re == 0.0) {
        RE(d) = 0.0;
        IM(d) = sinh(im);
        return d;
    }
    RE(d) = sin(re) * cosh(im);
    if (im == 0.0)
        IM(d) = im;
    else
        IM(d) = cos(re) * sinh(im);
    return d;

}
#line 1268 "src/pmc/complex.c"


PMC*
Parrot_Complex_cos(Interp* interpreter, PMC* pmc)
#line 1208 "src/pmc/complex.pmc"
{
    FLOATVAL re, im;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    re = RE(pmc);
    im = IM(pmc);
    if (re == 0.0) {
        RE(d) = cosh(im);
        IM(d) = 0.0;
        return d;
    }
    if (im == 0.0) {
        RE(d) = cos(re);
        IM(d) = 0.0;
        return d;
    }
    RE(d) = cos(re) * cosh(im);
    IM(d) = -1.0 * sin(re) * sinh(im);
    return d;

}
#line 1294 "src/pmc/complex.c"


PMC*
Parrot_Complex_tan(Interp* interpreter, PMC* pmc)
#line 1228 "src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    d = Parrot_Complex_sin(interpreter, pmc);
    e = Parrot_Complex_cos(interpreter, pmc);
    Parrot_Complex_i_divide_Complex(interpreter, d, e);
    return d;

}
#line 1309 "src/pmc/complex.c"


PMC*
Parrot_Complex_cot(Interp* interpreter, PMC* pmc)
#line 1237 "src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    d = Parrot_Complex_cos(interpreter, pmc);
    e = Parrot_Complex_sin(interpreter, pmc);
    Parrot_Complex_i_divide_Complex(interpreter, d, e);
    return d;

}
#line 1324 "src/pmc/complex.c"


PMC*
Parrot_Complex_sec(Interp* interpreter, PMC* pmc)
#line 1246 "src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    RE(d) = 1.0;
    IM(d) = 0.0;
    e = Parrot_Complex_cos(interpreter, pmc);
    Parrot_Complex_i_divide_Complex(interpreter, d, e);
    return d;

}
#line 1340 "src/pmc/complex.c"


PMC*
Parrot_Complex_csc(Interp* interpreter, PMC* pmc)
#line 1256 "src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    RE(d) = 1.0;
    IM(d) = 0.0;
    e = Parrot_Complex_sin(interpreter, pmc);
    Parrot_Complex_i_divide_Complex(interpreter, d, e);
    return d;

}
#line 1356 "src/pmc/complex.c"


PMC*
Parrot_Complex_asin(Interp* interpreter, PMC* pmc)
#line 1299 "src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    e = Parrot_Complex_multiply_Complex(interpreter, pmc, pmc, e);
    RE(e) = 1.0 - RE(e);
    IM(e) = -IM(e);
    d = Parrot_Complex_sqrt(interpreter, e);
    RE(d) -= IM(pmc);
    IM(d) += RE(pmc);
    d = Parrot_Complex_ln(interpreter, d);
    RE(e) = IM(d);
    IM(e) = -RE(d);
    return e;

}
#line 1377 "src/pmc/complex.c"


PMC*
Parrot_Complex_acos(Interp* interpreter, PMC* pmc)
#line 1314 "src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    e = Parrot_Complex_multiply_Complex(interpreter, pmc, pmc, e);
    RE(e) = 1.0 - RE(e);
    IM(e) = -IM(e);
    d = Parrot_Complex_sqrt(interpreter, e);
    RE(d) += IM(pmc);
    IM(d) -= RE(pmc);
    e = Parrot_Complex_ln(interpreter, d);
    RE(d) = IM(e) + 2.0 * atan(1);
    IM(d) = -RE(e);
    return d;

}
#line 1398 "src/pmc/complex.c"


PMC*
Parrot_Complex_atan(Interp* interpreter, PMC* pmc)
#line 1329 "src/pmc/complex.pmc"
{
    FLOATVAL re, im;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    re = RE(pmc);
    im = IM(pmc);
    RE(d) = re;
    IM(d) = 1 + im;
    RE(e) = -re;
    IM(e) = 1 - im;
    Parrot_Complex_i_divide_Complex(interpreter, d, e);
    d = Parrot_Complex_ln(interpreter, d);
    RE(e) = IM(d) / -2.0;
    IM(e) = RE(d) / 2.0;
    return e;

}
#line 1421 "src/pmc/complex.c"


PMC*
Parrot_Complex_acot(Interp* interpreter, PMC* pmc)
#line 1347 "src/pmc/complex.pmc"
{
    FLOATVAL a, b, aabb;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_atan(interpreter, d);
    return d;

}
#line 1439 "src/pmc/complex.c"


PMC*
Parrot_Complex_acsc(Interp* interpreter, PMC* pmc)
#line 1359 "src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_asin(interpreter, d);
    return d;

}
#line 1456 "src/pmc/complex.c"


PMC*
Parrot_Complex_asec(Interp* interpreter, PMC* pmc)
#line 1370 "src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_acos(interpreter, d);
    return d;

}
#line 1473 "src/pmc/complex.c"


PMC*
Parrot_Complex_sinh(Interp* interpreter, PMC* pmc)
#line 1401 "src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = sinh(a) * cos(b);
    IM(d) = cosh(a) * sin(b);
    return d;

}
#line 1489 "src/pmc/complex.c"


PMC*
Parrot_Complex_cosh(Interp* interpreter, PMC* pmc)
#line 1411 "src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = cosh(a) * cos(b);
    IM(d) = sinh(a) * sin(b);
    return d;

}
#line 1505 "src/pmc/complex.c"


PMC*
Parrot_Complex_tanh(Interp* interpreter, PMC* pmc)
#line 1421 "src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    d = Parrot_Complex_sinh(interpreter, pmc);
    e = Parrot_Complex_cosh(interpreter, pmc);
    Parrot_Complex_i_divide_Complex(interpreter, d, e);
    return d;

}
#line 1520 "src/pmc/complex.c"


PMC*
Parrot_Complex_coth(Interp* interpreter, PMC* pmc)
#line 1430 "src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    d = Parrot_Complex_tanh(interpreter, pmc);
    a = RE(d);
    b = IM(d);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    return d;

}
#line 1537 "src/pmc/complex.c"


PMC*
Parrot_Complex_csch(Interp* interpreter, PMC* pmc)
#line 1441 "src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    d = Parrot_Complex_sinh(interpreter, pmc);
    a = RE(d);
    b = IM(d);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    return d;

}
#line 1554 "src/pmc/complex.c"


PMC*
Parrot_Complex_sech(Interp* interpreter, PMC* pmc)
#line 1452 "src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    d = Parrot_Complex_cosh(interpreter, pmc);
    a = RE(d);
    b = IM(d);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    return d;

}
#line 1571 "src/pmc/complex.c"


PMC*
Parrot_Complex_asinh(Interp* interpreter, PMC* pmc)
#line 1491 "src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    RE(d) = IM(pmc);
    IM(d) = -RE(pmc);
    d = Parrot_Complex_asin(interpreter, d);
    RE(e) = -IM(d);
    IM(e) = RE(d);
    return e;

}
#line 1588 "src/pmc/complex.c"


PMC*
Parrot_Complex_acosh(Interp* interpreter, PMC* pmc)
#line 1502 "src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    d = Parrot_Complex_acos(interpreter, pmc);
    RE(e) = -IM(d);
    IM(e) = RE(d);
    return e;

}
#line 1603 "src/pmc/complex.c"


PMC*
Parrot_Complex_atanh(Interp* interpreter, PMC* pmc)
#line 1511 "src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    RE(d) = IM(pmc);
    IM(d) = -RE(pmc);
    d = Parrot_Complex_atan(interpreter, d);
    RE(e) = -IM(d);
    IM(e) = RE(d);
    return e;

}
#line 1620 "src/pmc/complex.c"


PMC*
Parrot_Complex_acoth(Interp* interpreter, PMC* pmc)
#line 1522 "src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    PMC *e = pmc_new(interpreter, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_atanh(interpreter, d);
    return d;

}
#line 1638 "src/pmc/complex.c"


PMC*
Parrot_Complex_acsch(Interp* interpreter, PMC* pmc)
#line 1534 "src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_asinh(interpreter, d);
    return d;

}
#line 1655 "src/pmc/complex.c"


PMC*
Parrot_Complex_asech(Interp* interpreter, PMC* pmc)
#line 1545 "src/pmc/complex.pmc"
{
    FLOATVAL a, b;
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    a = RE(pmc);
    b = IM(pmc);
    RE(d) = a/(a*a + b*b);
    IM(d) = -b/(a*a + b*b);
    d = Parrot_Complex_acosh(interpreter, d);
    return d;

}
#line 1672 "src/pmc/complex.c"


PMC*
Parrot_Complex_sqrt(Interp* interpreter, PMC* pmc)
#line 1601 "src/pmc/complex.pmc"
{
    PMC *d = pmc_new(interpreter, pmc->vtable->base_type);
    d = Parrot_Complex_ln(interpreter, pmc);
    RE(d) *= 0.5;
    IM(d) *= 0.5;
    d = Parrot_Complex_exp(interpreter, d);
    return d;

}
#line 1687 "src/pmc/complex.c"


void
Parrot_Complex_class_init(Parrot_Interp interp, int entry, int pass)
{
    const struct _vtable temp_base_vtable = {
        NULL,	/* namespace */
        enum_class_Complex,	/* base_type */
        NULL,	/* whoami */
        0, /* flags */
        NULL,   /* does_str */
        NULL,   /* isa_str */
        NULL,   /* class */
        NULL,   /* mro */
        Parrot_Complex_init,
        Parrot_Complex_init_pmc,
        Parrot_default_init_pmc_props,
        Parrot_Complex_instantiate,
        Parrot_default_new_from_string,
        Parrot_Complex_morph,
        Parrot_default_mark,
        Parrot_Complex_destroy,
        Parrot_default_namespace,
        Parrot_default_getprop,
        Parrot_default_setprop,
        Parrot_default_delprop,
        Parrot_default_getprops,
        Parrot_default_type,
        Parrot_default_type_keyed,
        Parrot_default_type_keyed_int,
        Parrot_default_type_keyed_str,
        Parrot_default_subtype,
        Parrot_default_name,
        Parrot_Complex_clone,
        Parrot_default_find_method,
        Parrot_default_hash,
        Parrot_Complex_get_integer,
        Parrot_Complex_get_integer_keyed,
        Parrot_default_get_integer_keyed_int,
        Parrot_Complex_get_integer_keyed_str,
        Parrot_Complex_get_number,
        Parrot_Complex_get_number_keyed,
        Parrot_Complex_get_number_keyed_int,
        Parrot_Complex_get_number_keyed_str,
        Parrot_default_get_bignum,
        Parrot_Complex_get_string,
        Parrot_default_get_repr,
        Parrot_default_get_string_keyed,
        Parrot_default_get_string_keyed_int,
        Parrot_default_get_string_keyed_str,
        Parrot_Complex_get_bool,
        Parrot_default_get_pmc,
        Parrot_Complex_get_pmc_keyed,
        Parrot_Complex_get_pmc_keyed_int,
        Parrot_Complex_get_pmc_keyed_str,
        Parrot_default_slice,
        Parrot_default_get_pointer,
        Parrot_default_get_pointer_keyed,
        Parrot_default_get_pointer_keyed_int,
        Parrot_default_get_pointer_keyed_str,
        Parrot_Complex_set_integer_native,
        Parrot_default_set_integer_same,
        Parrot_Complex_set_integer_keyed,
        Parrot_default_set_integer_keyed_int,
        Parrot_Complex_set_integer_keyed_str,
        Parrot_Complex_set_number_native,
        Parrot_default_set_number_same,
        Parrot_Complex_set_number_keyed,
        Parrot_Complex_set_number_keyed_int,
        Parrot_Complex_set_number_keyed_str,
        Parrot_default_set_bignum_int,
        Parrot_default_set_bignum_num,
        Parrot_default_set_bignum_str,
        Parrot_Complex_set_string_native,
        Parrot_default_assign_string_native,
        Parrot_default_set_string_same,
        Parrot_default_set_string_keyed,
        Parrot_default_set_string_keyed_int,
        Parrot_default_set_string_keyed_str,
        Parrot_default_set_bool,
        Parrot_Complex_set_pmc,
        Parrot_default_assign_pmc,
        Parrot_Complex_set_pmc_keyed,
        Parrot_default_set_pmc_keyed_int,
        Parrot_Complex_set_pmc_keyed_str,
        Parrot_default_set_pointer,
        Parrot_default_set_pointer_keyed,
        Parrot_default_set_pointer_keyed_int,
        Parrot_default_set_pointer_keyed_str,
        Parrot_default_elements,
        Parrot_default_pop_integer,
        Parrot_default_pop_float,
        Parrot_default_pop_string,
        Parrot_default_pop_pmc,
        Parrot_default_push_integer,
        Parrot_default_push_float,
        Parrot_default_push_string,
        Parrot_default_push_pmc,
        Parrot_default_shift_integer,
        Parrot_default_shift_float,
        Parrot_default_shift_string,
        Parrot_default_shift_pmc,
        Parrot_default_unshift_integer,
        Parrot_default_unshift_float,
        Parrot_default_unshift_string,
        Parrot_default_unshift_pmc,
        Parrot_default_splice,
        Parrot_default_increment,
        Parrot_default_decrement,
        Parrot_Complex_absolute,
        Parrot_Complex_i_absolute,
        Parrot_Complex_neg,
        Parrot_Complex_i_neg,
        Parrot_default_bitwise_not,
        Parrot_default_i_bitwise_not,
        Parrot_default_bitwise_nots,
        Parrot_default_i_bitwise_nots,
        Parrot_default_is_same,
        Parrot_default_logical_not,
        Parrot_default_i_logical_not,
        Parrot_default_substr,
        Parrot_default_substr_str,
        Parrot_default_exists_keyed,
        Parrot_default_exists_keyed_int,
        Parrot_default_exists_keyed_str,
        Parrot_default_defined,
        Parrot_default_defined_keyed,
        Parrot_default_defined_keyed_int,
        Parrot_default_defined_keyed_str,
        Parrot_default_delete_keyed,
        Parrot_default_delete_keyed_int,
        Parrot_default_delete_keyed_str,
        Parrot_default_nextkey_keyed,
        Parrot_default_nextkey_keyed_int,
        Parrot_default_nextkey_keyed_str,
        Parrot_default_get_iter,
        Parrot_Complex_invoke,
        Parrot_default_can,
        Parrot_default_does,
        Parrot_default_isa,
        Parrot_default_subclass,
        Parrot_default_get_attr,
        Parrot_default_get_attr_str,
        Parrot_default_set_attr,
        Parrot_default_set_attr_str,
        Parrot_default_get_class,
        Parrot_default_add_parent,
        Parrot_default_become_parent,
        Parrot_default_class_type,
        Parrot_default_add_method,
        Parrot_default_remove_method,
        Parrot_default_new_singleton,
        Parrot_default_get_anonymous_subclass,
        Parrot_default_freeze,
        Parrot_default_thaw,
        Parrot_default_thawfinish,
        Parrot_default_visit,
        Parrot_default_share
    };

    const  MMD_init _temp_mmd_init[] = {
        { MMD_ADD, 0, enum_type_PMC,
                    (funcptr_t) Parrot_Complex_add },
        { MMD_ADD, 0, enum_class_Complex,
                    (funcptr_t) Parrot_Complex_add_Complex },
        { MMD_ADD, 0, enum_type_INTVAL,
                    (funcptr_t) Parrot_Complex_add_int },
        { MMD_ADD, 0, enum_type_FLOATVAL,
                    (funcptr_t) Parrot_Complex_add_float },
        { MMD_I_ADD, 0, enum_type_PMC,
                    (funcptr_t) Parrot_Complex_i_add },
        { MMD_I_ADD, 0, enum_class_Complex,
                    (funcptr_t) Parrot_Complex_i_add_Complex },
        { MMD_I_ADD, 0, enum_type_INTVAL,
                    (funcptr_t) Parrot_Complex_i_add_int },
        { MMD_I_ADD, 0, enum_type_FLOATVAL,
                    (funcptr_t) Parrot_Complex_i_add_float },
        { MMD_SUBTRACT, 0, enum_type_PMC,
                    (funcptr_t) Parrot_Complex_subtract },
        { MMD_SUBTRACT, 0, enum_class_Complex,
                    (funcptr_t) Parrot_Complex_subtract_Complex },
        { MMD_SUBTRACT, 0, enum_type_INTVAL,
                    (funcptr_t) Parrot_Complex_subtract_int },
        { MMD_SUBTRACT, 0, enum_type_FLOATVAL,
                    (funcptr_t) Parrot_Complex_subtract_float },
        { MMD_I_SUBTRACT, 0, enum_type_PMC,
                    (funcptr_t) Parrot_Complex_i_subtract },
        { MMD_I_SUBTRACT, 0, enum_class_Complex,
                    (funcptr_t) Parrot_Complex_i_subtract_Complex },
        { MMD_I_SUBTRACT, 0, enum_type_INTVAL,
                    (funcptr_t) Parrot_Complex_i_subtract_int },
        { MMD_I_SUBTRACT, 0, enum_type_FLOATVAL,
                    (funcptr_t) Parrot_Complex_i_subtract_float },
        { MMD_MULTIPLY, 0, enum_type_PMC,
                    (funcptr_t) Parrot_Complex_multiply },
        { MMD_MULTIPLY, 0, enum_class_Complex,
                    (funcptr_t) Parrot_Complex_multiply_Complex },
        { MMD_MULTIPLY, 0, enum_type_INTVAL,
                    (funcptr_t) Parrot_Complex_multiply_int },
        { MMD_MULTIPLY, 0, enum_type_FLOATVAL,
                    (funcptr_t) Parrot_Complex_multiply_float },
        { MMD_I_MULTIPLY, 0, enum_type_PMC,
                    (funcptr_t) Parrot_Complex_i_multiply },
        { MMD_I_MULTIPLY, 0, enum_class_Complex,
                    (funcptr_t) Parrot_Complex_i_multiply_Complex },
        { MMD_I_MULTIPLY, 0, enum_type_INTVAL,
                    (funcptr_t) Parrot_Complex_i_multiply_int },
        { MMD_I_MULTIPLY, 0, enum_type_FLOATVAL,
                    (funcptr_t) Parrot_Complex_i_multiply_float },
        { MMD_DIVIDE, 0, enum_type_PMC,
                    (funcptr_t) Parrot_Complex_divide },
        { MMD_DIVIDE, 0, enum_class_Complex,
                    (funcptr_t) Parrot_Complex_divide_Complex },
        { MMD_DIVIDE, 0, enum_type_INTVAL,
                    (funcptr_t) Parrot_Complex_divide_int },
        { MMD_DIVIDE, 0, enum_type_FLOATVAL,
                    (funcptr_t) Parrot_Complex_divide_float },
        { MMD_I_DIVIDE, 0, enum_type_PMC,
                    (funcptr_t) Parrot_Complex_i_divide },
        { MMD_I_DIVIDE, 0, enum_class_Complex,
                    (funcptr_t) Parrot_Complex_i_divide_Complex },
        { MMD_I_DIVIDE, 0, enum_type_INTVAL,
                    (funcptr_t) Parrot_Complex_i_divide_int },
        { MMD_I_DIVIDE, 0, enum_type_FLOATVAL,
                    (funcptr_t) Parrot_Complex_i_divide_float },
        { MMD_POW, 0, enum_type_PMC,
                    (funcptr_t) Parrot_Complex_pow },
        { MMD_POW, 0, enum_class_Complex,
                    (funcptr_t) Parrot_Complex_pow_Complex },
        { MMD_EQ, 0, enum_type_PMC,
                    (funcptr_t) Parrot_Complex_is_equal },
        { MMD_EQ, 0, enum_class_Complex,
                    (funcptr_t) Parrot_Complex_is_equal_Complex }
    };
    /*  Dynamic PMCs need the runtime type
	which is passed in entry to class_init.
    */
    if (pass == 0) {
        /*
         * create vtable - clone it - we have to set a few items
         */
        struct _vtable *vt_clone =
            Parrot_clone_vtable(interp, &temp_base_vtable);

        vt_clone->whoami = CONST_STRING(interp, "Complex");
        vt_clone->isa_str = CONST_STRING(interp, "Complex");
        vt_clone->does_str = CONST_STRING(interp, "scalar");
        interp->vtables[entry] = vt_clone;
    } 
    else { /* pass */
        /* setup MRO and _namespace */
        Parrot_create_mro(interp, entry);
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_ln),
                "ln", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_exp),
                "exp", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_sin),
                "sin", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_cos),
                "cos", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_tan),
                "tan", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_cot),
                "cot", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_sec),
                "sec", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_csc),
                "csc", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_asin),
                "asin", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_acos),
                "acos", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_atan),
                "atan", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_acot),
                "acot", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_acsc),
                "acsc", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_asec),
                "asec", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_sinh),
                "sinh", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_cosh),
                "cosh", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_tanh),
                "tanh", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_coth),
                "coth", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_csch),
                "csch", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_sech),
                "sech", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_asinh),
                "asinh", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_acosh),
                "acosh", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_atanh),
                "atanh", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_acoth),
                "acoth", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_acsch),
                "acsch", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_asech),
                "asech", "PJO");
        enter_nci_method(interp, entry,
                F2DPTR(Parrot_Complex_sqrt),
                "sqrt", "PJO");
        /* class_init */
    
#define N_MMD_INIT (sizeof(_temp_mmd_init)/sizeof(_temp_mmd_init[0]))
        Parrot_mmd_register_table(interp, entry,
            _temp_mmd_init, N_MMD_INIT);
    } /* pass */
} /* Parrot_Complex_class_init */


/*

=back

=cut

*/

/*
 * Local variables:
 * c-indentation-style: bsd
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
*/
