/*
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 *
 * This file is generated automatically from 'src/pmc/timer.pmc'
 * by tools/build/pmc2c.pl.
 *
 * Any changes made here will be lost!
 *
 */

#line 1 "src/pmc/timer.pmc"
/*
Copyright: 2001-2003 The Perl Foundation.  All Rights Reserved.
$Id: timer.pmc 11355 2006-01-26 12:23:21Z leo $

=head1 NAME

src/pmc/timer.pmc - Timer

=head1 SYNOPSIS

    new P0, .Timer
    set P0[.PARROT_TIMER_SEC], I_seconds        # whole seconds
    set P0[.PARROT_TIMER_USEC], I_micro seconds # and/or micro seconds
    set P0[.PARROT_TIMER_NSEC], N_seconds_frac  # or fraction in seconds
    set P0[.PARROT_TIMER_REPEAT], I_repeat      # 0 = one shot ...
    set P0[.PARROT_TIMER_HANDLER], P_sub        # set handler sub PMC
    invoke P0                                   # start timer
    set P0[.PARROT_TIMER_RUNNING], 1            # same

    new P0, .Timer, P1                          # set everything

    set I0, P0[.PARROT_TIMER_SEC]               # query current timer status
    set N0, P0[.PARROT_TIMER_NSEC]
    ...
    set P0[.PARROT_TIMER_RUNNING], 0            # turn timer off


=head1 DESCRIPTION

This is the Timer base class

Running timers are kept in a linked list. Each timer has a tick count,
which gets decremented if the system timer expires. If the tick count
reaches zero, the timer handler gets invoked via Parrot's event handling
code.

The Timer resolution is operating system dependent. It's only guaranteed
that the Timer will fire some time after the programmed interval.

The Timer stops after invoking the handler (repeat+1) times. To create a
Timer that will run forever, set "repeat" to -1. Turning the Timer off
preserves set values; the Timer is not destroyed.

When setting both C<PARROT_TIMER_SEC> and C<PARROT_TIMER_USEC> it must
be done in that sequence, whole seconds first. If a timer is constructed
with an initializer and it should be run immediately, the
C<PARROT_TIMER_RUNNING> value has to be in the last key of the
initializer.

=head2 Functions

=over 4

=cut

*/

#include "parrot/parrot.h"

/* interface constants */
/* &gen_from_enum(timer.pasm) */
typedef enum {
    PARROT_TIMER_SEC,
    PARROT_TIMER_USEC,
    PARROT_TIMER_NSEC,
    PARROT_TIMER_REPEAT,
    PARROT_TIMER_RUNNING,
    PARROT_TIMER_HANDLER,
    PARROT_TIMER_MAX
} parrot_timer_enum_t;
/* &end_gen */

/*

=item C<static void
add_timer(Interp* interpreter, PMC *pmc)>

Adds a new timer event to the event queue.

=cut

*/

static void
add_timer(Interp* interpreter, PMC *pmc)
{
    parrot_timer_event *t = PMC_struct_val(pmc);
    if (!PMC_IS_NULL(t->sub))
        Parrot_new_timer_event(interpreter, pmc, t->abs_time,
            t->interval, t->repeat, t->sub, EVENT_TYPE_TIMER);
}

/*

=item C<static void
del_timer(Interp* interpreter, PMC *pmc)>

Deletes the timer event.

=cut

*/

static void
del_timer(Interp* interpreter, PMC *pmc)
{
   Parrot_del_timer_event(interpreter, pmc);
}


#line 123 "src/pmc/timer.c"
#include "parrot/parrot.h"
#include "parrot/extend.h"
#include "parrot/dynext.h"
#include "pmc_timer.h"
#include "pmc_default.h"
#include "timer.str"

void
Parrot_Timer_init(Interp* interpreter, PMC* pmc)
#line 123 "src/pmc/timer.pmc"
{
    /*
     * t->abs_time actually is relative, its converted to an
     * absolute time, when the timer is started
     */
    parrot_timer_event *t =
        mem_sys_allocate_zeroed(sizeof(parrot_timer_event));
    PMC_struct_val(pmc) = t;
    PObj_active_destroy_SET(pmc);
    ++interpreter->arena_base->num_early_DOD_PMCs;

}
#line 146 "src/pmc/timer.c"


void
Parrot_Timer_init_pmc(Interp* interpreter, PMC* pmc, PMC *init)
#line 145 "src/pmc/timer.pmc"
{
    INTVAL n = VTABLE_get_integer(interpreter, init);
    INTVAL i;
    INTVAL val;
    FLOATVAL nval;
    PMC *pval;

    Parrot_Timer_init(interpreter, pmc);
    for (i = 0; i < n; i += 2) {
        INTVAL key = VTABLE_get_integer_keyed_int(interpreter, init, i);
        switch (key) {
            case PARROT_TIMER_SEC:
            case PARROT_TIMER_USEC:
            case PARROT_TIMER_REPEAT:
            case PARROT_TIMER_RUNNING:
                val =
                VTABLE_get_integer_keyed_int(interpreter, init, i+1);
                Parrot_Timer_set_integer_keyed_int(interpreter, pmc, key, val);
                break;
            case PARROT_TIMER_NSEC:
                nval =
                VTABLE_get_number_keyed_int(interpreter, init, i+1);
                Parrot_Timer_set_number_keyed_int(interpreter, pmc, key, nval);
                break;
            case PARROT_TIMER_HANDLER:
                pval =
                VTABLE_get_pmc_keyed_int(interpreter, init, i+1);
                Parrot_Timer_set_pmc_keyed_int(interpreter, pmc, key, pval);
                break;
        }
    }

}
#line 185 "src/pmc/timer.c"


void
Parrot_Timer_destroy(Interp* interpreter, PMC* pmc)
#line 188 "src/pmc/timer.pmc"
{
    parrot_timer_event *t = PMC_struct_val(pmc);
    del_timer(interpreter, pmc);
    mem_sys_free(t);

}
#line 197 "src/pmc/timer.c"


INTVAL
Parrot_Timer_get_integer_keyed_int(Interp* interpreter, PMC* pmc, INTVAL key)
#line 204 "src/pmc/timer.pmc"
{
    parrot_timer_event *t = PMC_struct_val(pmc);

    switch(key) {
        case PARROT_TIMER_SEC:
            return (INTVAL)t->abs_time;
        case PARROT_TIMER_USEC:
            return (INTVAL)( (t->abs_time - (INTVAL)t->abs_time) *1000000.0 );
        case PARROT_TIMER_REPEAT:
            return (INTVAL) t->repeat;
        case PARROT_TIMER_RUNNING:
            return 0;
    }
    return -1;

}
#line 219 "src/pmc/timer.c"


FLOATVAL
Parrot_Timer_get_number_keyed_int(Interp* interpreter, PMC* pmc, INTVAL key)
#line 249 "src/pmc/timer.pmc"
{
    parrot_timer_event *t = PMC_struct_val(pmc);

    switch(key) {
        case PARROT_TIMER_NSEC:
            return t->abs_time;
    }
    return -1.0;

}
#line 235 "src/pmc/timer.c"


PMC*
Parrot_Timer_get_pmc_keyed_int(Interp* interpreter, PMC* pmc, INTVAL key)
#line 230 "src/pmc/timer.pmc"
{
    parrot_timer_event *t = PMC_struct_val(pmc);
    switch(key) {
        case PARROT_TIMER_HANDLER:
            return t->sub;
    }
    return PMCNULL;

}
#line 250 "src/pmc/timer.c"


void
Parrot_Timer_set_integer_keyed_int(Interp* interpreter, PMC* pmc, INTVAL key, INTVAL value)
#line 269 "src/pmc/timer.pmc"
{
    parrot_timer_event *t = PMC_struct_val(pmc);

    switch(key) {
        case PARROT_TIMER_SEC:
            t->abs_time = value;
            break;
        case PARROT_TIMER_USEC:
            t->abs_time += value / 1000000.0;
            break;
        case PARROT_TIMER_REPEAT:
            t->repeat = value;
            break;
        case PARROT_TIMER_RUNNING:
            if (value) {
                add_timer(interpreter, pmc);
            }
            else {
                del_timer(interpreter, pmc);
            }
            break;
    }

}
#line 280 "src/pmc/timer.c"


void
Parrot_Timer_set_number_keyed_int(Interp* interpreter, PMC* pmc, INTVAL key, FLOATVAL value)
#line 337 "src/pmc/timer.pmc"
{
    parrot_timer_event *t = PMC_struct_val(pmc);

    switch(key) {
        case PARROT_TIMER_NSEC:
            t->abs_time = value;
            break;
    }

}
#line 296 "src/pmc/timer.c"


void
Parrot_Timer_set_pmc_keyed_int(Interp* interpreter, PMC* pmc, INTVAL key, PMC* value)
#line 303 "src/pmc/timer.pmc"
{
    parrot_timer_event *t = PMC_struct_val(pmc);
    switch(key) {
        case PARROT_TIMER_HANDLER:
            t->sub = value;
            break;
    }

}
#line 311 "src/pmc/timer.c"


void*
Parrot_Timer_invoke(Interp* interpreter, PMC* pmc, void *next)
#line 322 "src/pmc/timer.pmc"
{
    add_timer(interpreter, pmc);
    return next;

}
#line 322 "src/pmc/timer.c"


void
Parrot_Timer_class_init(Parrot_Interp interp, int entry, int pass)
{
    const struct _vtable temp_base_vtable = {
        NULL,	/* namespace */
        enum_class_Timer,	/* base_type */
        NULL,	/* whoami */
        0, /* flags */
        NULL,   /* does_str */
        NULL,   /* isa_str */
        NULL,   /* class */
        NULL,   /* mro */
        Parrot_Timer_init,
        Parrot_Timer_init_pmc,
        Parrot_default_init_pmc_props,
        Parrot_default_instantiate,
        Parrot_default_new_from_string,
        Parrot_default_morph,
        Parrot_default_mark,
        Parrot_Timer_destroy,
        Parrot_default_namespace,
        Parrot_default_getprop,
        Parrot_default_setprop,
        Parrot_default_delprop,
        Parrot_default_getprops,
        Parrot_default_type,
        Parrot_default_type_keyed,
        Parrot_default_type_keyed_int,
        Parrot_default_type_keyed_str,
        Parrot_default_subtype,
        Parrot_default_name,
        Parrot_default_clone,
        Parrot_default_find_method,
        Parrot_default_hash,
        Parrot_default_get_integer,
        Parrot_default_get_integer_keyed,
        Parrot_Timer_get_integer_keyed_int,
        Parrot_default_get_integer_keyed_str,
        Parrot_default_get_number,
        Parrot_default_get_number_keyed,
        Parrot_Timer_get_number_keyed_int,
        Parrot_default_get_number_keyed_str,
        Parrot_default_get_bignum,
        Parrot_default_get_string,
        Parrot_default_get_repr,
        Parrot_default_get_string_keyed,
        Parrot_default_get_string_keyed_int,
        Parrot_default_get_string_keyed_str,
        Parrot_default_get_bool,
        Parrot_default_get_pmc,
        Parrot_default_get_pmc_keyed,
        Parrot_Timer_get_pmc_keyed_int,
        Parrot_default_get_pmc_keyed_str,
        Parrot_default_slice,
        Parrot_default_get_pointer,
        Parrot_default_get_pointer_keyed,
        Parrot_default_get_pointer_keyed_int,
        Parrot_default_get_pointer_keyed_str,
        Parrot_default_set_integer_native,
        Parrot_default_set_integer_same,
        Parrot_default_set_integer_keyed,
        Parrot_Timer_set_integer_keyed_int,
        Parrot_default_set_integer_keyed_str,
        Parrot_default_set_number_native,
        Parrot_default_set_number_same,
        Parrot_default_set_number_keyed,
        Parrot_Timer_set_number_keyed_int,
        Parrot_default_set_number_keyed_str,
        Parrot_default_set_bignum_int,
        Parrot_default_set_bignum_num,
        Parrot_default_set_bignum_str,
        Parrot_default_set_string_native,
        Parrot_default_assign_string_native,
        Parrot_default_set_string_same,
        Parrot_default_set_string_keyed,
        Parrot_default_set_string_keyed_int,
        Parrot_default_set_string_keyed_str,
        Parrot_default_set_bool,
        Parrot_default_set_pmc,
        Parrot_default_assign_pmc,
        Parrot_default_set_pmc_keyed,
        Parrot_Timer_set_pmc_keyed_int,
        Parrot_default_set_pmc_keyed_str,
        Parrot_default_set_pointer,
        Parrot_default_set_pointer_keyed,
        Parrot_default_set_pointer_keyed_int,
        Parrot_default_set_pointer_keyed_str,
        Parrot_default_elements,
        Parrot_default_pop_integer,
        Parrot_default_pop_float,
        Parrot_default_pop_string,
        Parrot_default_pop_pmc,
        Parrot_default_push_integer,
        Parrot_default_push_float,
        Parrot_default_push_string,
        Parrot_default_push_pmc,
        Parrot_default_shift_integer,
        Parrot_default_shift_float,
        Parrot_default_shift_string,
        Parrot_default_shift_pmc,
        Parrot_default_unshift_integer,
        Parrot_default_unshift_float,
        Parrot_default_unshift_string,
        Parrot_default_unshift_pmc,
        Parrot_default_splice,
        Parrot_default_increment,
        Parrot_default_decrement,
        Parrot_default_absolute,
        Parrot_default_i_absolute,
        Parrot_default_neg,
        Parrot_default_i_neg,
        Parrot_default_bitwise_not,
        Parrot_default_i_bitwise_not,
        Parrot_default_bitwise_nots,
        Parrot_default_i_bitwise_nots,
        Parrot_default_is_same,
        Parrot_default_logical_not,
        Parrot_default_i_logical_not,
        Parrot_default_substr,
        Parrot_default_substr_str,
        Parrot_default_exists_keyed,
        Parrot_default_exists_keyed_int,
        Parrot_default_exists_keyed_str,
        Parrot_default_defined,
        Parrot_default_defined_keyed,
        Parrot_default_defined_keyed_int,
        Parrot_default_defined_keyed_str,
        Parrot_default_delete_keyed,
        Parrot_default_delete_keyed_int,
        Parrot_default_delete_keyed_str,
        Parrot_default_nextkey_keyed,
        Parrot_default_nextkey_keyed_int,
        Parrot_default_nextkey_keyed_str,
        Parrot_default_get_iter,
        Parrot_Timer_invoke,
        Parrot_default_can,
        Parrot_default_does,
        Parrot_default_isa,
        Parrot_default_subclass,
        Parrot_default_get_attr,
        Parrot_default_get_attr_str,
        Parrot_default_set_attr,
        Parrot_default_set_attr_str,
        Parrot_default_get_class,
        Parrot_default_add_parent,
        Parrot_default_become_parent,
        Parrot_default_class_type,
        Parrot_default_add_method,
        Parrot_default_remove_method,
        Parrot_default_new_singleton,
        Parrot_default_get_anonymous_subclass,
        Parrot_default_freeze,
        Parrot_default_thaw,
        Parrot_default_thawfinish,
        Parrot_default_visit,
        Parrot_default_share
    };
    if (pass == 0) {
        /*
         * create vtable - clone it - we have to set a few items
         */
        struct _vtable *vt_clone =
            Parrot_clone_vtable(interp, &temp_base_vtable);

        vt_clone->whoami = CONST_STRING(interp, "Timer");
        vt_clone->isa_str = CONST_STRING(interp, "Timer");
        vt_clone->does_str = CONST_STRING(interp, "event");
        interp->vtables[entry] = vt_clone;
    } 
    else { /* pass */
        /* setup MRO and _namespace */
        Parrot_create_mro(interp, entry);
        /* class_init */
    
    } /* pass */
} /* Parrot_Timer_class_init */


/*

=back

=head1 HISTORY

Initial proposal by leo 2003.07.15

Initial revision 2003.07.17

Switch to events 2004.01.07

=cut

*/

/*
 * Local variables:
 * c-indentation-style: bsd
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
*/
